/*
 * jQuery.imgzoom v1.0 图片放大镜
 * Author  :   Leo
 * QQ      :   657448678
 */
;(function($, win, doc, undefined){
	$.fn.imgzoom = function(options){
		// 共用变量
		var opts = $.extend(true, {}, $.fn.imgzoom.defaultOpts, options),
			lensAuto = opts.lensAuto, scale = opts.scale, 
			lensW = opts.lensWidth, lensH =	opts.lensHeight,lensL = 0, lensT = 0,
			viewerW , viewerH, imgL, imgT, imgW, imgH, maxW, maxH, /*halfW, halfH,*/ bImgW, bImgH,
			$viewer = $('#imgzoom-viewer'),
			moveAllowed = false, timeoutId = null;
			calculate = function(dImg){ /*相关参数的计算*/
				var $img = $(dImg);
				imgL = $img.offset().left;
				imgT = $img.offset().top;
				imgW = dImg.offsetWidth;
				imgH = dImg.offsetHeight;

				if(lensAuto){
					if(imgW > imgH)	{
						lensW = opts.lensWidth;
						lensH = Math.ceil(imgH * lensW / imgW);
					}else{
						lensH = opts.lensHeight;
						lensW = Math.ceil(imgW * lensH / imgH);
					}	
				}

				viewerW = lensW * scale;
				viewerH = lensH * scale;
				maxW = imgW - lensW;
				maxH = imgH - lensH;
				// halfW = lensW / 2;
				// halfH = lensH / 2;
			};
		return this.each(function(){
			var img = this,
				$img = $(img),
				$lens = $img.next('span.imgzoom-lens'),
				imgParent = null,
				$imgParent = null, bigImg = null;

			calculate(img);	// 初始化计算
			
			// 初始化HTML结构
			if($img.data('imgzoom-mark')) return true;
			$img.data('imgzoom-mark', true);
			
			imgParent = doc.createElement('span');
			imgParent.className = 'imgzoom-wrap';
			imgParent.style.cssText = ';display:inline-block;position:relative;max-width:100%;max-height:100%;line-height:0;font-size:0;';
			img.parentNode.insertBefore(imgParent,img);
			imgParent.appendChild(img);
			
			$imgParent =$(imgParent);
			
			// 放大镜头
			if($lens.length === 0){
				$lens = $('<span/>').addClass('imgzoom-lens').css({
							cursor:'move',
							width: lensW-2,
							height: lensH - 2,
							backgroundColor:'#E2340C',
							opacity: 0.3,
							border:'1px solid red',
							position:'absolute',
							left: lensL,top: lensT,
							display: 'none'
						}).appendTo(imgParent);
			}

			// 放大显示框
			if($viewer.length === 0){
				$viewer = $('<div id="imgzoom-viewer"/>').appendTo(doc.body).css({
					display: 'none',
					background: '#fff url('+opts.loading+') no-repeat center center',
					width: viewerW - 2,
					height: viewerH - 2,
					overflow:'hidden',
					border:'1px solid #aaa',
					position:'absolute',
					zIndex: '10000'
				});
			}

			// 放大镜事件
			$imgParent.bind('mouseenter', function(event){
				moveAllowed = false;
				calculate(img); // 再次计算

				timeoutId = setTimeout(function(){
					// 加载大图
					bigImg = new Image();
					bigImg.style.display = 'none';
					bigImg.style.position = 'absolute';
					
					bigImg.onload = function(){
						$viewer[0].style.backgroundImage = '';
						this.style.display = 'block';

						bImgW = this.width;
						bImgH = this.height;
						if(lensAuto){
							if(bImgW <= viewerW) viewerW = bImgW;
							if(bImgH <= viewerH) viewerH = bImgH;	
						}

						if(bImgW <= viewerW && bImgH<= viewerH){
							viewerW = bImgW;
							viewerH = bImgH;
							$lens.hide();	
						}

						$viewer.css({width: viewerW, height: viewerH});
						if(bImgW >= imgW || bImgH >= imgH){
							moveAllowed = true;
						}
					};
					bigImg.onerror = function(){
						$viewer[0].style.backgroundImage = '';
						$viewer[0].innerHTML = '<p style="background:#fff;padding:10px 0;text-align:center;color:#aaa;line-height:'+(viewerH-20)+'px;">图片加载失败</p>';
					};

					if($img.attr('iz-bigsrc')){
						$viewer.html('').append(bigImg);
						$viewer[0].style.backgroundImage = 'url('+opts.loading+')';
						$lens.css({width: lensW-2, height: lensH - 2}).show();
						$viewer.css({
							width: viewerW, 
							height: viewerH,
							left: imgL + imgW + 20, 
							top: imgT
						}).show();
						bigImg.src = $img.attr('iz-bigsrc');
					}
					
				}, 50);
				
			}).bind('mouseleave', function(){
				clearTimeout(timeoutId);
				$lens.hide();
				$viewer.hide();
			}).bind('mousemove', function(event){
				// 放大镜头移动的范围
				lensL = Math.floor(event.pageX - imgL - lensW / 2);
				lensT = Math.floor(event.pageY - imgT - lensH / 2);
				lensL < 0 && (lensL = 0);
				lensL > maxW && (lensL = maxW);
				lensT < 0 && (lensT = 0);
				lensT > maxH && (lensT = maxH);


				// 万恶的IE mousemove先于mouseenter触发
				if($lens && $lens[0]){
					$lens[0].style.left = lensL + 'px';
					$lens[0].style.top = lensT + 'px';

				}

				if(moveAllowed && bigImg && bigImg.style != undefined){
					if(bImgW > viewerW) bigImg.style.left = -(lensL * ((bImgW - viewerW) / (imgW - lensW))) + 'px';
					if(bImgH > viewerH) bigImg.style.top = -(lensT * ((bImgH - viewerH) / (imgH - lensH))) + 'px';
				}
				
			});
		});	
	};
	
	$.fn.imgzoom.defaultOpts = {
		lensAuto: true,					// 镜头高宽比例跟图片一致
		lensWidth: 120,					// 镜头宽
		lensHeight: 120,				// 镜头高
		scale: 4,						// 放大几倍
		loading: 'images/loading.gif' 	// 大图loading图
	};


})(jQuery, window, document);